clc, clear all, close all

L = 520; %%% Numero di dati dell'intera sequenza

N = 500; %%% Numero di dati di lavoro

%%%
%%% Parametri del modello FIR
%%%

disp('Vettore vero dei parametri del FIR:')

theta0 = [0.4 0.2]'

pause

Bz = [0 theta0'];
Az = [1 0.0 0.0];

n = 2; %%% Numero di regressori del FIR

std_e = 0.3; %%% Deviazione standard dell'errore di equazione (EE)

randn('state',0);      %%% Generazione dell'ingresso deterministico u(t)
u = randn(L,1);
u = (u - mean(u))/std(u);

yv = filter(Bz,Az,u);  %%% Generazione dell'uscita 
 
randn('state',1);
e = randn(L,1);
e = std_e * (e - mean(e))/std(e); %%% Generazione dell'ingresso stocastico e(t)

y = yv + e;  %%% Generazione dell'uscita y(t) con errore d'equazione e(t)

Phi = newhank(u,N,n); %%% Matrice di Hankel dell'ingresso, di dimensione N x n
Y   = y(n+1:N+n);         %%% Vettore delle uscite Y costruito da y(t), di dimensione N x 1 

disp('Vettore dei parametri stimato da algoritmo LS:')

theta_hat_LS = inv(Phi' * Phi) * Phi' * Y  %%% Formula LS

pause

Ym = Phi * theta_hat_LS; %%% Uscita prevista

epserr = Y - Ym; %%% Errore di previsione

sigma_e = (epserr' * epserr)/N; %%% Stima della varianza dell'errore d'equazione

disp('Matrice di covarianza della stima LS')

cov_theta_hat_LS = sigma_e * inv(Phi' * Phi) %%% Stima della matrice di covarianza della stima

pause

disp('Deviazione standard dei parametri del FIR')

std_theta_hat_LS = sqrt(diag(cov_theta_hat_LS))

pause

disp('Errore relativo dei parametri del FIR (%)')

100 * std_theta_hat_LS./theta_hat_LS

pause

%%%
%%% Simulazione e grafici ingressi e uscite del modello FIR
%%%

figure, subplot(211), plot([1:L]',u,'k-')
                      title('Ingresso del modello FIR'), ylabel('u(t)')
                      xlabel('Campioni')

        subplot(212), plot([1:L]',y,'k-')
                      title('Uscita del modello FIR'), ylabel('y(t)')
                      xlabel('Campioni')

pause                      

figure, subplot(211), plot([1:N]',Y,'k-',[1:N]',Ym,'c--')
                      title('Uscita vera (nero) e misurata (magenta)'), ylabel('Y e Y_m')
                      xlabel('Campioni')

        subplot(212), plot([1:N]',epserr,'r-')
                      title('Errore di previsione'), ylabel('\epsilon')
                      xlabel('Campioni')


pause                      



nNind = 1;

for Ni=25:25:1.5e4,
    
    randn('state',0);      %%% Generazione dell'ingresso deterministico u(t)
    u = randn(Ni+20,1);
    u = (u - mean(u))/std(u);

    yv = filter(Bz,Az,u);  %%% Generazione dell'uscita 
 
    randn('state',1);
    e = randn(Ni+20,1);
    e = std_e * (e - mean(e))/std(e); %%% Generazione dell'ingresso stocastico e(t)

    y = yv + e;  %%% Generazione dell'uscita y(t) con errore d'equazione e(t)

    Phi = newhank(u,Ni,n); %%% Matrice di Hankel dell'ingresso, di dimensione N x n
    Y   = y(n+1:Ni+n);         %%% Vettore delle uscite Y costruito da y(t), di dimensione N x 1 

    theta_hat_LS_N(:,nNind) = inv(Phi' * Phi) * Phi' * Y;  %%% Formula LS
    nN(1,nNind) = Ni; 
    nNind = nNind + 1;
    
end;    

decim = 10;

figure, subplot(211), plot(nN,ones(size(nN))*theta0(2),'k-',nN(1:decim:end),theta_hat_LS_N(1,1:decim:end),'bd')
        title('Consistenza della stima di {\theta}_N'), ylabel('b_2'),
        xlabel('Numero di Campioni N')
        
        subplot(212), plot(nN,ones(size(nN))*theta0(1),'k-',nN(1:decim:end),theta_hat_LS_N(2,1:decim:end),'rd')               
        title('Consistenza della stima di {\theta}_N'), ylabel('b_1'),
        xlabel('Numero di Campioni N \cap')


return